/*
   Copyright (c) 2009-2016, Jack Poulson
   All rights reserved.

   This file is part of Elemental and is under the BSD 2-Clause License,
   which can be found in the LICENSE file in the root directory, or at
   http://opensource.org/licenses/BSD-2-Clause
*/
#include <El-lite.hpp>
#include <El/blas_like/level1.hpp>
#include <El/matrices.hpp>

namespace El {

template<typename T>
void MakeIdentity( Matrix<T>& I )
{
    EL_DEBUG_CSE
    Zero( I );
    FillDiagonal( I, T(1) );
}

template<typename T>
void MakeIdentity( AbstractDistMatrix<T>& I )
{
    EL_DEBUG_CSE
    Zero( I );
    FillDiagonal( I, T(1) );
}

template<typename T>
void Identity( Matrix<T>& I, Int m, Int n )
{
    EL_DEBUG_CSE
    I.Resize( m, n );
    MakeIdentity( I );
}

template<typename T>
void Identity( AbstractDistMatrix<T>& I, Int m, Int n )
{
    EL_DEBUG_CSE
    I.Resize( m, n );
    MakeIdentity( I );
}


#define PROTO(T) \
  template void MakeIdentity( Matrix<T>& I ); \
  template void MakeIdentity( AbstractDistMatrix<T>& I ); \
  template void Identity( Matrix<T>& I, Int m, Int n ); \
  template void Identity( AbstractDistMatrix<T>& I, Int m, Int n );

#define EL_ENABLE_DOUBLEDOUBLE
#define EL_ENABLE_QUADDOUBLE
#define EL_ENABLE_QUAD
#define EL_ENABLE_BIGINT
#define EL_ENABLE_BIGFLOAT
#define EL_ENABLE_HALF
#include <El/macros/Instantiate.h>

} // namespace El
